package xdoclet.maven;

import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.jelly.XMLOutput;
import org.apache.commons.jelly.JellyTagException;
import org.apache.commons.jelly.MissingAttributeException;
import org.apache.commons.jelly.tags.ant.TaskSource;

import org.apache.maven.jelly.MavenJellyContext;
import org.apache.maven.jelly.tags.BaseTagSupport;

   /**
    * Bulk property setter. 
    * Similar to org.apache.commons.jelly.tags.ant.SetPropertyTag, but processes
    * sets of properties with a given prefix.
    * Optionally, a set of comma-delimited suffixes can be specified. In this 
    * case there is no scanning of the context for the matching prefix, but 
    * the prefix is rather prepended to each value to produce search keys.
    * @author Florin Vancea (fvancea at maxiq dot com)
    */
public class SetPropertiesTag extends BaseTagSupport {

   /** Property: prefix.  */
   private String prefix = null;
   /** Property: includes */
   private String includes = null;

   public void setPrefix(String aPrefix) {
      if (aPrefix == null) prefix = null; // will throw MissingAttributeException
      else if (!aPrefix.endsWith(".")) prefix = aPrefix + ".";
      else prefix = aPrefix;
   }

   public String getPrefix() {
      return prefix;
   }

   public void setIncludes(String aIncludes) {
      includes = aIncludes;
   }

   public String getIncludes() {
      return includes;
   }

   public void doTag( XMLOutput output ) 
         throws MissingAttributeException, JellyTagException {
      TaskSource tag = (TaskSource) findAncestorWithClass( TaskSource.class );
      if ( tag == null ) {
         throw new JellyTagException( "This tag must be nested within an Ant task tag" );
      }
      if (prefix == null) {
         throw new MissingAttributeException("prefix");
      }
      MavenJellyContext projectCtx = getMavenContext();
      //System.out.println("Prefix="+prefix);
      if (includes == null) {
         // scan all that match
         int prefLen = prefix.length();
         Map vars = Util.getCascadedVars(projectCtx.getProject(), prefix);
         Iterator iter = vars.entrySet().iterator();
         while (iter.hasNext()) {
            Map.Entry me = (Map.Entry) iter.next();
            String varName = (String) me.getKey();
            //System.out.println("Processing "+varName);
            if (!varName.startsWith(prefix)) continue;
            if (me.getValue() == null) continue;
            String propName = varName.substring(prefLen);
            if (propName.indexOf('.') >= 0) continue;
            //System.out.println("Setting "+propName+" to "+me.getValue());
            tag.setTaskProperty(propName, me.getValue());
         }
      } else {
         StringTokenizer sTok = new StringTokenizer(includes, ",", false);
         MavenJellyContext pluginCtx = 
            projectCtx.getProject().getPluginContext(Util.PLUGIN_ID);
         while (sTok.hasMoreTokens()) {
            String propName = prefix + sTok.nextToken();
            Object propVal = null;
            if (pluginCtx.getVariable(propName) != null) 
               propVal = pluginCtx.getVariable(propName);
            // this is probably redundant...
            if (projectCtx.getVariable(propName) != null) 
               propVal = projectCtx.getVariable(propName);
            if (propVal == null) continue;
            tag.setTaskProperty(propName, propVal);
         }
      }
   }

}